const PREFETCH = '__PREFETCH__';

function generateHash(obj) {
    return Object.keys(obj).sort().reduce(function (hash, k) {
        const v = obj[k];
        if (typeof v === 'undefined') {
            return hash;
        }
        const str = `${k}:${typeof v === 'object' ? generateHash(v) : v}`;
        for (let i = 0; i < str.length; i++) {
            hash = (hash << 5) - hash + str.charCodeAt(i);
        }
        return hash;
    }, 0).toString(36);
}

function getCache(params) {
    try {
        // @ts-expect-error
        if (!window[PREFETCH] || !Object.keys(window[PREFETCH]).length)
            return null;
        const { url, method, header, body } = params;
        const options = { url, method, header, body };
        const key = generateHash(options);
        // @ts-expect-error
        const cache = window[PREFETCH][key];
        if (cache && cache.count > 0) {
            cache.count -= 1;
            if (cache.count < 1) {
                // @ts-expect-error
                delete window[PREFETCH][key];
            }
            return cache.value;
        }
        return null;
    }
    catch (error) {
        return null;
    }
}

// @ts-nocheck
function formatSourceData() {
    return {
        cookie: document.cookie.split("; ").reduce(function (result, cookie) {
            const [name, value] = cookie.split("=");
            result[name] = decodeURIComponent(value);
            return result;
        }, {}),
        query: location.search
            .slice(1)
            .split("&")
            .reduce(function (result, item) {
            var parts = item.split("=");
            result[decodeURIComponent(parts[0])] = decodeURIComponent(parts[1] || "");
            return result;
        }, {}),
        hash: location.hash.slice(2),
        path: location.pathname,
    };
}

export { formatSourceData, getCache };
